package cmd

import (
	"bufio"
	"fmt"
	"io"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"strings"
	"sync"
)

type Crawler struct {
	wg   sync.WaitGroup
	path string
}

func NewCrawler() *Crawler {
	return &Crawler{}
}

func (c *Crawler) Start() {
	c.path = output
	if folderTitleUrl != "" {
		chapterTitle := FindChapterTitle(folderTitleUrl, chapter)
		title := strings.Join([]string{
			"第",
			strconv.Itoa(chapter),
			"话-",
			chapterTitle,
		}, "")
		c.path = filepath.Join(output, title) // 组装章节路径
	}

	err := IfPathNotExistDoMkdir(c.path)
	if err != nil {
		fmt.Println("输出目录创建失败:", err)
		return
	}

	c.wg.Add(max)
	for i := 1; i <= max; i++ {
		go c.do(i)
	}

	c.wg.Wait()
}

func (c *Crawler) do(num int) {
	defer c.wg.Done()

	// 兼容未携带斜杆的地址
	if url[0] != '/' {
		url = "/" + url
	}

	numString := strconv.Itoa(num)
	urlSlice := strings.Split(url, "_")          // 取URL组装
	fileSlice := strings.Split(urlSlice[1], ".") // 取后缀名
	fileName := strings.Join([]string{           // 组装文件名
		numString,
		fileSlice[1],
	}, ".")

	urlNum := numString
	if fileSlice[0][0] == '0' { // 序号首位为0，则链接补0
		urlNum = fmt.Sprintf("%02d", num)
	}

	imgUrl := strings.Join([]string{ // 组装图片URL
		host,
		urlSlice[0],
		"_",
		urlNum,
		".",
		fileSlice[1],
	}, "")
	fmt.Println("获取图片:" + imgUrl)

	resp, err := http.Get(imgUrl)
	if err != nil {
		fmt.Println(fileName, "图片获取失败:", err)
		return
	}
	defer func(Body io.ReadCloser) {
		err := Body.Close()
		if err != nil {
			fmt.Println(fileName, "http请求关闭失败:", err)
			return
		}
	}(resp.Body)

	reader := bufio.NewReaderSize(resp.Body, 32*1024)
	file, err := os.Create(c.path + "/" + fileName)
	if err != nil {
		fmt.Println(fileName, "图片创建失败:", err)
		return
	}
	defer func(file *os.File) {
		err := file.Close()
		if err != nil {
			fmt.Println(fileName, "文件流关闭失败:", err)
			return
		}
	}(file)

	_, err = io.Copy(file, reader)
	if err != nil {
		fmt.Println(fileName, "图片输出失败:", err)
		return
	}
}
